@@ -66,11 +66,11 @@ import logging
 
 import tomlkit
 import pluginlib
-from PySide2 import QtCore, QtWidgets
-from PySide2.QtCore import Signal         #pylint: disable=no-name-in-module
+from PySide6 import QtCore, QtWidgets
+from PySide6.QtCore import Signal
 
-import configuration as config            #pylint: disable=import-error
-import hardware as hw                     #pylint: disable=import-error
+import configuration as config
+import hardware as hw
 
 
 #: Default settings to place in configuration file if not existing already.
@@ -463,7 +463,7 @@ config.add_defaults(tomlkit.parse(_default_config))
 devices = [None for _ in config.data["hardware"]["delay"]]
 
 # Bypass this code block if module was imported by sphinx documentation generation utility
-if not "sphinx-build" in os.path.basename(sys.argv[0]):
+if not "sphinx" in sys.argv[0]:
     # Look for valid plugin classes
     try:
         plugin_paths = [ os.path.join(d, "delay") for d in config.data["directories"]["plugins"] ]
